# react-notify-resize
React Element that notifies you whenever it is resized - by any means.

It can be used as a function that takes a component and returns a `NotifyResize`.
Or as a high order component that takes as prop `factory` a component.

You can be notified of resize by two means:
- inside your component if you define a `onResize` method it will be called when resized
- outside, NotifyResize takes a prop `onResize` calledwhen element resized

## Usage

Example:
```jsx
import notifyResize from 'react-notify-resize'

class MyComponent extends React.Component {
  
  render(){
    // the element you want to listen for resize must have position relative
    return <div style={{position: 'relative'}}>
      {
        // Include this helper inside the element you want to listen for resize
        // It renders two divs, with position absolute
        this.props.resizeTool
      }
    </div>
  }
      
  // will be called on resize if it is defined
  onResize(){
    // do something on resize
  }
}

const MyNotifiedComponent =  notifyResize(MyComponent)

<MyNotifiedComponent onResize={/* event called when elementchanges dimension */}
```

### Function
```jsx
import MyComponent from './MyComponent'
import notifyResize from 'react-notify-resize'

const MyNotifiedComponent = notifyResize(MyComponent)
```

### Component
```jsx
import MyComponent from './MyComponent'
import { NotifyResize } from 'react-notify-resize'

class SomeComponent extends React.Component {
  render(){
    return <NotifyResize factory={MyComponent} />
  }
}
```

## Props
Returned Component from `notifyResize`

Property | Type | Default | Description
----|---|---|---
`onResize`| Function | - | called when component changes


`NotifyResize` Component

Property | Type | Default | Description
----|---|---|---
`factory`| React Component | - | component to decorate
