import React, { PropTypes } from 'react'

const notifyResizeStyle = {
    position: 'absolute',
    top: 0,
    left: 0,
    width: '100%',
    height: '100%',
    zIndex: -1,
    overflow: 'auto',
    display: 'block',
    pointerEvents: 'none',
    opacity: 0
  }

class NotifyResize extends React.Component {
  constructor(props){
    super(props)
 
    this.state = {
      resizeToolWidth: 0,
      resizeToolHeight: 0,
      notifyResizeWidth: 0,
      notifyResizeHeight: 0
    }

    this.checkResize = this.checkResize.bind(this)
  }

  render() {
    const props = this.props
    const Component = this.props.factory


    const resizeTool = <div 
      ref="notifyResize"
      className="notifyResize"
      style={notifyResizeStyle}
      onScroll={this.checkResize}
    >
        <div 
          ref="resizeTool"
          className="resizeTool" 
          style={{
            position: 'absolute',
            top: 0,
            left: 0,
            width: this.state.resizeToolWidth,
            height: this.state.resizeToolHeight
          }}
        />
      </div>
    

    return <Component 
      {...props} 
      factory={null} 
      resizeTool={resizeTool} 
      ref="component"
    />
  }

  componentDidMount(){
    this.resetResizeTool()
  }

  getDimensions(){
    const resizeTool = this.refs.resizeTool
    const notifyResize = this.refs.notifyResize

    return {
      resizeToolWidth: resizeTool.offsetWidth,
      resizeToolHeight: resizeTool.offsetHeight,
      notifyResizeWidth: notifyResize.offsetWidth,
      notifyResizeHeight: notifyResize.offsetHeight
    }    
  }

  resetResizeTool(){
    // check if they are monted
    if(!this.refs.resizeTool && !this.refs.notifyResize) {
      console.error(
          'For notifyResize to work you must render resizeTool from {props.resizeTool}'
        )
      return false
    }

    this.setDimensions()
    this.scrollToBottom()
  }

  setDimensions(){
    const {
      notifyResizeWidth,
      notifyResizeHeight
    } = this.getDimensions()
  

    // Resize tool will be bigger than it's parent by 1 pixel in each direction
    this.setState({
      notifyResizeWidth,
      notifyResizeHeight,
      resizeToolWidth: notifyResizeWidth + 2,
      resizeToolHeight: notifyResizeHeight + 2
    })

  }

  scrollToBottom(){
    // so the scroll moves when element resizes
    
    if (this.refs.notifyResize) {
      setTimeout(() => {
        this.refs.notifyResize.scrollTop = this.refs.notifyResize.scrollHeight
        this.refs.notifyResize.scrollLeft = this.refs.notifyResize.scrollWidth
      }, 0)
    }
  }


  checkResize(){
    const {
      notifyResizeWidth,
      notifyResizeHeight
    } = this.getDimensions()

    if(
      notifyResizeWidth !== this.state.notifyResizeWidth ||
      notifyResizeHeight !== this.state.notifyResizeHeight
      ){
      // reset resizeToolDimensions
      this.onResize()
      this.resetResizeTool()
    }
  }

  onResize(){
    if (this.props.onResize) {
      this.props.onResize()
    }

    if (this.refs.component.onResize) {
      this.refs.component.onResize()
    }
  }


}

NotifyResize.propTypes = {
  onResize: PropTypes.func
}

const notifyResize = Component => class extends React.Component {
  render() {
    return <NotifyResize factory={Component} {...this.props} />
  }
}

export default notifyResize

export {
  NotifyResize
}
